package com.persagy.ztkedgeclouddatasecurity.datasafety;

import cn.shuibo.config.SecretKeyConfig;
import cn.shuibo.util.Base64Util;
import cn.shuibo.util.RSAUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

@Service
public class EncryptInputMessageService implements  EncryptInputMessageInterface{
    private Logger log = LoggerFactory.getLogger(this.getClass());
    @Value("${rsa.encrypt.open}")
    private boolean encrypt;
    @Autowired
    private SecretKeyConfig secretKeyConfig;
    private static ThreadLocal<Boolean> encryptLocal = new ThreadLocal<>();
    @Value("${rsa.encrypt.open}")
    private  String converterType;

    public boolean supports(String converterType) {
        encrypt = false;
        if (secretKeyConfig.isOpen()) {
            encrypt = true;
        }
        return encrypt;
    }
    class Model {

        @JSONField(jsonDirect=true)
        public String content;
    }

    @Override
    public  Object beforeBodyWrite(Object body) {
        // EncryptResponseBodyAdvice.setEncryptStatus(false);
        // Dynamic Settings Not Encrypted
        Boolean status = encryptLocal.get();
        if (null != status && !status) {
            encryptLocal.remove();
            return body;
        }
        if (encrypt) {
            String publicKey = secretKeyConfig.getPublicKey();
            try {
                Model model =new Model();
                model.content=body.toString();
                String content = JSONObject.toJSONString(model);

                if (!StringUtils.hasText(publicKey)) {
                    throw new NullPointerException("Please configure rsa.encrypt.privatekeyc parameter!");
                }
                byte[] data = content.getBytes();
                byte[] encodedData = RSAUtil.encrypt(data, publicKey);
                String result = Base64Util.encode(encodedData);
                if(secretKeyConfig.isShowLog()) {
                    log.info("Pre-encrypted data：{}，After encryption：{}", content, result);
                }
                return result;
            } catch (Exception e) {
                log.error("Encrypted data exception", e);
            }
        }
        return body;
    }

}

