package com.persagy.ztkedgeclouddatasecurity.kafka;

import com.alibaba.fastjson.JSONObject;
import com.persagy.ztkedgeclouddatasecurity.entity.NettyMessage;
import com.persagy.ztkedgeclouddatasecurity.netty.cloud.NettyClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.kafka.support.KafkaHeaders;
import org.springframework.messaging.handler.annotation.Header;
import org.springframework.stereotype.Component;

import java.util.Optional;


@Component
@Slf4j
@ConditionalOnProperty(prefix = "spring", name="location", havingValue="Cloud")
public class CloudKafkaConsumer {

    @Autowired
    public NettyClient nettyClient;
    //@KafkaListener(topics = KafkaProducer.TOPIC_TEST, groupId = KafkaProducer.TOPIC_GROUP1)
    @KafkaListener(topicPattern = "Edge_.*")
    public void topicList(ConsumerRecord<?, ?> record, Acknowledgment ack, @Header(KafkaHeaders.RECEIVED_TOPIC) String topic) {
      Optional message = Optional.ofNullable(JSONObject.parseObject(record.value().toString()));
        if (message.isPresent()) {
            Object msg = message.get();
            log.info("Cloud 消费了： Topic:" + topic + ",Message:" + msg);
            JSONObject.parseObject(JSONObject.toJSONString(msg)).getObject("msg", NettyMessage.class);
            try {
                nettyClient.sendMessage(JSONObject.parseObject(JSONObject.toJSONString(msg)).getObject("msg", NettyMessage.class));
                if (ack != null) {
                    ack.acknowledge();
                }
            } catch (InterruptedException e) {
               log.error(e.getMessage(),e);
            }




        }

    }

}
