package com.persagy.ztkedgeclouddatasecurity.netty;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.ztkedgeclouddatasecurity.entity.ChattingUser;
import com.persagy.ztkedgeclouddatasecurity.entity.NettyMessage;
import com.persagy.ztkedgeclouddatasecurity.entity.NettyMessageQueue;
import com.persagy.ztkedgeclouddatasecurity.kafka.EdgeKafkaProducer;
import com.persagy.ztkedgeclouddatasecurity.netty.edge.NettyServer;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.util.concurrent.GlobalEventExecutor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class MsgHandler extends SimpleChannelInboundHandler<NettyMessage<JSONObject>> {
    /**
     * 保留所有与服务器建立连接的channel对象
     */
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    public static ChannelGroup channelGroup = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss:SSS");//格式化
    BlockingQueue<String> messageQueue = new LinkedBlockingQueue<>(1024 * 1024);
//    @Autowired
//    private EncryptInputMessageService encryptInputMessageService;
//    @Autowired
//    private DecryptInputMessageService decryptInputMessageService;

    public ChattingUser chattingUser;

    private final static ExecutorService executorService = new ThreadPoolExecutor(
            Runtime.getRuntime().availableProcessors(),
            Runtime.getRuntime().availableProcessors() * 2, 60, TimeUnit.SECONDS,
            new LinkedBlockingQueue(50000));

    private EdgeKafkaProducer edgeKafkaProducer;



    public MsgHandler(ChattingUser chattingUser, EdgeKafkaProducer edgeKafkaProducer) {
        this.chattingUser =chattingUser;
        this.edgeKafkaProducer=edgeKafkaProducer;
        this.edgeKafkaProducer.producerMsg(messageQueue);

    }

    public String GetRemoteIP(String str){
        String result ="";
        String regEx="((2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(2[0-4]\\d|25[0-5]|[01]?\\d\\d?)";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(str);
        while (m.find()) {
            result = m.group();
            // System.out.println(result);
            //break;   加break则提取string中的一个IP
        }
        return result;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, NettyMessage<JSONObject> msg) throws Exception {
        log.info("收到[{}]消息:{}", ctx.channel().remoteAddress(), msg);
        //super.channelRead(ctx, msg);
        // nConnection.decrementAndGet();
        InetSocketAddress inteSocket = (InetSocketAddress) ctx.channel().localAddress();
        String localip = inteSocket.getAddress().getHostAddress();
        String localport = String.valueOf(inteSocket.getPort());
        System.out.println(
                "server ip: " + localip + "  Server port "+ localport
        );

        // System.out.println(">>>>>"+nConnection.decrementAndGet());
        String time = sdf.format(new Date());
//            ByteBuf in = (ByteBuf) msg;
//            System.out.println(
//                    "server receive: " + in.toString(CharsetUtil.UTF_8)
//            );
//            String str = in.toString(CharsetUtil.UTF_8);
        JSONObject obj =new JSONObject();
        obj.put("port",localport);
        obj.put("msg",msg);
        obj.put("time",time);
        // List<ChattingUser> list= listInfoClump.getListInfo();
        //EncryptInputMessage.beforeBodyWrite(obj);
        // for (int j=0;j<list.size();j++){
        //    if (Integer.valueOf(localport) == list.get(j).getPort() ){
        obj.put("userid",chattingUser.getUserId());
        obj.put("targetAddress",chattingUser.getIp());

        //   }
        // }
        System.out.println(">>>>"+obj.toString());
        NettyMessageQueue.getNettyMessageQueue().produce(obj.toString());
       // messageQueue.offer(obj.toString(), 1000, TimeUnit.MICROSECONDS);
       // chattingUser.setMessageTcpSendQueue(messageQueue);
       // edgeKafkaProducer.producerMsg(messageQueue);


        //System.out.println("dataEncrypt>>>"+encryptInputMessageService.beforeBodyWrite(obj).toString());
        // decryptInputMessageService.DecryptMsgInputMessage(encryptInputMessageService.beforeBodyWrite(obj).toString());
        //System.out.println("dataDecrypt>>>"+ decryptInputMessageService.DecryptMsgInputMessage(encryptInputMessageService.beforeBodyWrite(obj).toString()));

//            if (!messageQueue.isEmpty()&&m==0 ){
//                m++;
//                edgeKafkaProducer.setQueue(messageQueue);
//            }

        /**
         *
         * ctx.write(in);
         * Bytebuf并不适合直接重用，必须新建才能写回客户端，
         * 否则将会报io.netty.util.IllegalReferenceCountException: refCnt: 0, decrement: 1
         * 原因是这是因为Netty4有引用计数器的原因，自从Netty 4开始，对象的生命周期由它们的引用计数（reference counts）管理，
         * 而不是由垃圾收集器（garbage collector）管理了。ByteBuf是最值得注意的，它使用了引用计数来改进分配内存和释放内存的性能。
         * 在我们创建ByteBuf对象后，它的引用计数是1，当你释放（release）引用计数对象时，它的引用计数减1，
         * 如果引用计数为0，这个引用计数对象会被释放（deallocate）,并返回对象池。
         * 当尝试访问引用计数为0的引用计数对象会抛出IllegalReferenceCountException异常：
         * 或者在该Bytebuf 在复用前需要调用retain(),将计数器置为1
         */
        // ByteBuf pong = Unpooled.copiedBuffer(str.getBytes());
        //ctx.write(pong);

        // System.out.println(GetRemoteIP(String.valueOf(ctx.channel().remoteAddress()))+"接受到..Server received: " + in.toString(CharsetUtil.UTF_8));
        System.out.println(GetRemoteIP(String.valueOf(ctx.channel().remoteAddress()))+"接受到..Server received: " + msg);
        Channel channel =ctx.channel();

        //System.out.println( channel.pipeline().get("id"));

        String ip = GetRemoteIP(String.valueOf(ctx.channel().remoteAddress()));
        if (ip.equals(chattingUser.getIp())) {
            System.out.println("Certification Ip List>>>>"+ip);
        }else {
            System.out.println("No_Certification Ip List>>>>"+ip +" Messageinfo>>>"+ channel.id().asLongText());
        }

//        if (!messageQueue.isEmpty()){
//
//            edgeKafkaProducer.setQueue(messageQueue);
//        }
//        executorService.submit(new Runnable() {
//            @Autowired
//            private EdgeKafkaProducer edgeKafkaProducer;
//            @Override
//            public void run() {
//                if (!messageQueue.isEmpty()){
//                   edgeKafkaProducer.setQueue(messageQueue);
//                }
//            }
//        });
//        executorService.shutdown();
//        if (executorService.awaitTermination(10, TimeUnit.SECONDS)) {
//            logger.info("shutdown success");
//        } else {
//            logger.info("failed to handle messages");
//        }
        // while (true){

        //  }
        // ctx.write(in);
        //ctx.write(msg);
        // ctx.flush();


    }


    @Override
    public void channelRegistered(ChannelHandlerContext ctx) throws Exception {
        SocketAddress socketAddress = ctx.channel().remoteAddress();
        String remoteAddress = socketAddress.toString();
        log.warn("--某个客户端绑定地址:[{}]--", remoteAddress);
    }


    /**
     * 在读取操作期间，有异常抛出时会调用。
     *
     * @param ctx
     * @param cause
     * @throws Exception
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        log.error("netty 连接异常" + ctx.channel().remoteAddress(), cause);
        ctx.close();
    }

    /**
     * 新增连接
     */
    @Override
    public void handlerAdded(ChannelHandlerContext ctx) throws Exception {
        //NettyServer.socketChannelMap.put(ctx.channel().remoteAddress().toString(), ctx.channel());
        channelGroup.add(ctx.channel());
        log.info("当前连接数：[{}],新建立连接为[{}]...", channelGroup.size(), ctx.channel().remoteAddress().toString());
        if (CollectionUtil.isNotEmpty(channelGroup)) {
            for (Channel channel : channelGroup) {
                log.info("已有连接地址为[{}]", channel.remoteAddress().toString());
            }
        }
        super.handlerAdded(ctx);
    }

    /**
     * 断开连接
     */
    @Override
    public void handlerRemoved(ChannelHandlerContext ctx) throws Exception {
        //chanel可以理解成Connection
        NettyServer.removeChannel(ctx);
        //下面这行代码Netty会自动调用
        channelGroup.remove(ctx.channel());
        super.handlerRemoved(ctx);
        log.warn("----客户端[{}] ----离开", ctx.channel().remoteAddress().toString());
    }

    /**
     * 该方法只会在通道建立时调用一次，连接生效
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.warn("[{}]:channelActive", ctx.channel().remoteAddress());
        super.channelActive(ctx);
    }

    /**
     * 连接是否有效
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        ctx.channel().close();
        log.warn("[{}]:channelInactive", ctx.channel().remoteAddress());
        super.channelInactive(ctx);
    }
}

