package com.persagy.ztkedgeclouddatasecurity.netty.cloud;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import com.alibaba.fastjson.JSONObject;
import com.persagy.ztkedgeclouddatasecurity.entity.NettyMessage;
import com.persagy.ztkedgeclouddatasecurity.entity.alerm.CommonConst;
import com.persagy.ztkedgeclouddatasecurity.kafka.CloudKafkaConsumer;
import com.persagy.ztkedgeclouddatasecurity.kafka.CloudKafkaProducer;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;

import java.util.Arrays;
import java.util.concurrent.TimeUnit;

@Slf4j
//@ConditionalOnProperty(prefix = "spring", name="location", havingValue="Cloud")
public class CenterClientHandler extends SimpleChannelInboundHandler<NettyMessage> {
    // Sleep 5 seconds before a reconnection attempt.
    static final int RECONNECT_DELAY = Integer.parseInt(System.getProperty("reconnectDelay", "5"));
    // Reconnect when the server sends nothing for 10 seconds.
    private static final int READ_TIMEOUT = Integer.parseInt(System.getProperty("readTimeout", "10"));

    private CloudKafkaConsumer cloudKafkaConsumer;
    private final NettyClient nettyClient;

    private static NettyMessage  nettyMessage;
    private  String projectID;

     private CloudKafkaProducer cloudKafkaProducer;

    @Value("${spring.location}")
    private String location;



    public CenterClientHandler(CloudKafkaConsumer cloudKafkaConsumer, NettyClient nettyClient,String projectID,CloudKafkaProducer cloudKafkaProducer) {
        this.cloudKafkaConsumer = cloudKafkaConsumer;
        this.nettyClient = nettyClient;
        this.projectID = projectID;
        this.cloudKafkaProducer = cloudKafkaProducer;
        // 加项目的标识

    }

    /**
     * 在到服务器的连接已经建立之后将被调用
     * 当客户端主动链接服务端的链接后，这个通道就是活跃的了。也就是客户端与服务端建立了通信通道并且可以传输数据
     *
     * @param ctx
     * @throws Exception
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("Connected to: {}", ctx.channel().remoteAddress());
        //建立连接
        nettyClient.sendMessage(new NettyMessage("",200,"Pj4403070003","ZS"));
        //启动的时候发送消息,获取全部报警定义
        NettyMessage nettyMessage = new NettyMessage("",4,"Pj4403070003","ZS");
        JSONObject content = new JSONObject();
        content.put("groupCode", CommonConst.groupCode);
        content.put("projectId", CommonConst.projectId);
        nettyMessage.setContent(Arrays.asList(content));
        nettyClient.sendMessage(nettyMessage);

        //内存中缓冲的未发送成功的数据发送出去
        iniAlarmResult(ctx);
    }

    private void iniAlarmResult(ChannelHandlerContext ctx) {

    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, NettyMessage msg) throws Exception {
        //接收msg消息{与上一章节相比，此处已经不需要自己进行解码}
        log.info("Client received: {}", msg);
        System.out.println(">>>>>"+projectID);
        JSONObject cloudboj= new JSONObject();
        cloudboj.put("projectid",projectID);
        cloudboj.put("msg",msg);
        try {
            if ("Edge".equals(location)){

            }else {
                cloudKafkaProducer.CloudproducerMsg(cloudboj);
            }

            TimeInterval timer = DateUtil.timer();
            handlerMsg(ctx, msg);
            log.info("处理消息时间[{}]",timer.interval()  +">>>>>>"+ ctx.channel());
        } catch (Exception e) {
            log.error("channelRead", e);
        }
    }

    private void handlerMsg(ChannelHandlerContext channelHandlerContext, NettyMessage msg) throws Exception {

    }

    /**
     * 在处理过程中引发异常时被调用
     * 抓住异常，当发生异常的时候，可以做一些相应的处理，比如打印日志、关闭链接
     *
     * @param ctx
     * @param cause
     * @throws Exception
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        log.info("exceptionCaught", cause);
        ctx.close();
    }

    /**
     * 通道处于非活跃状态动作，该方法只会在失效时调用一次
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        //客户端自己不正常情况下自己在重连一次
        log.info("Disconnected from: " + ctx.channel().remoteAddress());

    }


    @Override
    public void channelUnregistered(final ChannelHandlerContext ctx) throws Exception {
        log.info("channelUnregistered and reconnecting to: {}:{} ", RECONNECT_DELAY, CommonConst.inetHost, CommonConst.inetPort);
        ctx.channel().eventLoop().schedule(new Runnable() {
            @Override
            public void run() {
                nettyClient.reConnect();
            }
        }, RECONNECT_DELAY, TimeUnit.SECONDS);
    }

}
