package com.persagy.ztkedgeclouddatasecurity.netty.cloud;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.persagy.ztkedgeclouddatasecurity.entity.NettyMessage;
import com.persagy.ztkedgeclouddatasecurity.entity.NettyMessageQueue;
import com.persagy.ztkedgeclouddatasecurity.kafka.CloudKafkaConsumer;
import com.persagy.ztkedgeclouddatasecurity.kafka.CloudKafkaProducer;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

@Component
@Slf4j
//@ConditionalOnProperty(prefix = "spring", name="location", havingValue="Cloud")
public class NettyClient {
    public static Channel channel;
    public Bootstrap bootstrap = new Bootstrap();
    @Autowired
    private  CloudKafkaConsumer cloudKafkaConsumer;

    @Autowired
    private  CloudKafkaProducer cloudKafkaProducer;

    @Value("${listener.cloud1}")
    private String Connectinfo;

    @PostConstruct
    public void connect() {
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        bootstrap.group(workerGroup);
        bootstrap.channel(NioSocketChannel.class);
        bootstrap.option(ChannelOption.AUTO_READ, true);


        String[] connectInfoArr=Connectinfo.split(",");
        for (int i=0;i<connectInfoArr.length;i++){

            String ip= connectInfoArr[i].split(":")[2];
            int port = Integer.valueOf(connectInfoArr[i].split(":")[3]);
            String projectId = connectInfoArr[i].split(":")[3]+"_"+connectInfoArr[i].split(":")[1];
        try {
            bootstrap.handler(new CenterChannelInitializer(cloudKafkaConsumer,this,projectId,cloudKafkaProducer));
            ChannelFuture f = bootstrap.connect(ip, port).sync();
            channel = f.channel();
            // f.channel().closeFuture().sync();
        } catch (Exception e) {
            log.info("连接异常", e);
        } finally {
            //workerGroup.shutdownGracefully();
        }
        }
    }


    public void reConnect() {
        // 加入断线后自动重连监听器
        String[] connectInfoArr=Connectinfo.split(",");
        for (int i=0;i<connectInfoArr.length;i++){
            String ip= connectInfoArr[i].split(":")[2];
            int port = Integer.valueOf(connectInfoArr[i].split(":")[3]);
            channel = bootstrap.connect(ip, port).addListener(new ChannelFutureListener() {
                @Override
                public void operationComplete(ChannelFuture future) throws Exception {

                    if (future.cause() != null) {
                        log.info("Failed to connect: {}", future.cause());
                    }
                }
            }).channel();


        }

    }


    public void sendMessage(NettyMessage msg) throws InterruptedException {
        log.info("给云端发送数据：[{}]", msg);
        if (channel.isWritable()) {
            try {
                //!!!!!!注意，NettyMessage<T>是有泛型的，如果把发送一个数据，接收方必须有一个和<T>的全限定类名完全一样的实体类，
                //发送方不应该限制接收方，所有使用JSONObject.parseObject(msg.toString(), NettyMessage.class)操作让泛型擦除掉
                channel.writeAndFlush(JSONObject.parseObject(msg.toString(), new TypeReference<NettyMessage<JSONObject>>(){}));
            } catch (Exception e) {
                log.error("发送数据异常,放入缓冲队列中", e);
                NettyMessageQueue.getNettyMessageQueue().cloudproduce(msg);
                channel.close();
            }
        } else {
            log.warn("云端netty不可写，放入缓冲队列中[{}]", msg);
            NettyMessageQueue.getNettyMessageQueue().cloudproduce(msg);
        }
    }

    public void sendMessageNotCheck(NettyMessage msg){
        try {
            channel.writeAndFlush(JSONObject.parseObject(msg.toString(), NettyMessage.class));
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }


}
