package com.persagy.ztkedgeclouddatasecurity.netty.edge;

import cn.hutool.core.collection.CollectionUtil;
import com.persagy.ztkedgeclouddatasecurity.entity.ChattingUser;
import com.persagy.ztkedgeclouddatasecurity.kafka.EdgeKafkaProducer;
import com.persagy.ztkedgeclouddatasecurity.netty.MyChannelInitializer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

@Component
@Slf4j
@ConditionalOnProperty(prefix = "spring", name="location", havingValue="Edge")
public class NettyServer {

    public static Map<String, Set<Channel>> socketChannelMap = new ConcurrentHashMap<>();
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    @Value("${listener.port}")
    private String listenerPort;

    @Value("${spring.profiles.active}")
    private String active;
    @Value("${spring.location}")
    private String location;

    private ChannelFuture[] ChannelFutures = null;
    private List<ChattingUser> list = new ArrayList<>();
    @Autowired
    private EdgeKafkaProducer edgeKafkaProducer;

    public static synchronized void removeChannel(ChannelHandlerContext ctx) {
        Channel channel = ctx.channel();
        log.warn("删除channel[{}]", ctx.channel().remoteAddress().toString());
        Set<Map.Entry<String, Set<Channel>>> channelList = NettyServer.socketChannelMap.entrySet();
        for (Map.Entry<String, Set<Channel>> channelEntry : channelList) {
            Set<Channel> channels = channelEntry.getValue();
            if (CollectionUtil.isNotEmpty(channels)) {
                Iterator<Channel> it = channels.iterator();
                while (it.hasNext()) {
                    Channel ch = it.next();
                    if (ch == channel) {
                        it.remove();
                        log.warn("----项目ID[{}],地址[{}] ----离开---", channelEntry.getKey(), channel.remoteAddress().toString());
                    }
                }
                if (CollectionUtil.isNotEmpty(channels)) {
                    log.warn("项目[{}]剩余客户端数量[{}]", channelEntry.getKey(), channels.size());
                    NettyServer.socketChannelMap.put(channelEntry.getKey(), channels);
                } else {
                    log.warn("项目[{}]已经全部下线", channelEntry.getKey());
                    NettyServer.socketChannelMap.remove(channelEntry.getKey());
                }
            }
        }
    }

    @PostConstruct
    private void start() {

        //多端口绑定
        String[] ipPortArr = listenerPort.split(",");
        // ListInfoClump listInfoClump =new ListInfoClump();

        for (int i = 1; i <= ipPortArr.length; i++) {

            ChattingUser chattingUser = new ChattingUser();
            chattingUser.setUserId(ipPortArr[i - 1].split(":")[1]);
            String ip = "";
            if ("Cloud".equals(location) && (!"dev".equals(active))) {
                chattingUser.setIp(ipPortArr[i - 1].split(":")[1]);
                ip = ipPortArr[i - 1].split(":")[1];
            } else {
                chattingUser.setIp(ipPortArr[i - 1].split(":")[2]);
                ip = ipPortArr[i - 1].split(":")[2];
            }

            if ("zkt-proj-alarm".equals(ipPortArr[i - 1].split(":")[1])) {
                ServerBootstrap serverBootstrap = new ServerBootstrap();
                EventLoopGroup bossGroup = new NioEventLoopGroup();
                NioEventLoopGroup workerGroup = new NioEventLoopGroup();
                serverBootstrap.group(bossGroup, workerGroup);
                serverBootstrap.channel(NioServerSocketChannel.class);
                serverBootstrap.childOption(ChannelOption.SO_REUSEADDR, true);
                int port = Integer.valueOf(ipPortArr[i - 1].split(":")[3]);
                chattingUser.setPort(port);
                list.add(chattingUser);
                serverBootstrap.childHandler(new MyChannelInitializer(chattingUser, edgeKafkaProducer));
                ChannelFuture channelFuture = serverBootstrap.bind(port);
                if (ChannelFutures == null) {
                    ChannelFutures = new ChannelFuture[8];
                }
                ChannelFutures[i] = channelFuture;
                channelFuture.addListener(future -> {
                    if (future.isSuccess()) {
                        System.out.println("Started success,port:" + port + ">>>" + channelFuture.channel().pipeline().channel().id().asLongText());
                    } else {
                        System.out.println("Started Failed,port:" + port + ">>>" + channelFuture.channel().pipeline().channel().id().asLongText());
                    }
                });

            }

        }

    }
}
